## libraries and constants ####
library(ggplot2)
library(scales)
library(egg)
library(grid)

plotTheme2 = 
  theme_bw(base_size=9) +
  theme(#panel.grid.major=element_blank(),
        panel.grid.minor=element_blank(),
        axis.text=element_text(size=9#,
                               #colour="black"
                  ))

driverAC = 500 # the cost of the company for attracting a driver
riderAC = 15 # the cost of the company for attracting a rider


## the model ($6 charged, 100 rides per month, 60% answering) ####

# riderCost = 25 # the cost of a ride for a rider
driverCost = 6 # the cost of a ride for a driver (paid to the company); will it be variable later?
driverRideRate = 100 # the monthly rate of rides; will it become variable later, as a function of driverCost?
driverQuitRate = 0.2 # the monthly rate of driver quit
happyRiderQuitRate = 0.1 # the monthly rate of a rider who has been served so far 
sadRiderQuitRate = 0.33 # the monthly rate of a rider who has not been served at least once in the past
repsCount = 100

set.seed(112)
dataPre = NULL
for (rep in 1:repsCount)
{
  print(rep)
  
  companyMoney = 0 # the budget of the company
  drivers = 0 # the initial number of drivers
  riders = 0 # the initial number of riders
  monthsVec = 1:12
  initAcqDrivers = 100 # will it be a variable later?
  initAcqRiders = 16600 # will it be a variable later?
  
  # at the end of the previous month, acquisitions are assumed to have occurred
  acqDrivers = initAcqDrivers # this for testing; 
  acqRiders = initAcqRiders # this for testing;
  initCAC = (acqDrivers * driverAC) + (acqRiders * riderAC)
  companyMoney = companyMoney - initCAC # update the company's money with the CAC
  happyRiders = rep(1, acqRiders)
  
  
  # the monthly loop
  for(month in monthsVec)
  {
    drivers = drivers + acqDrivers
    riders = riders + acqRiders
    
    # assuming that each acquired rider will call once per month
    
    # calls during the month
    yesRides = min( # the number of answered calls
      riders, # if there are enough drivers, all riders are answered
      drivers * driverRideRate
    )
    noRides = riders - yesRides
    noRiders = sample( # select randomly
      x=1:riders, # which of the riders
      size=noRides, # were not answered
      replace=FALSE
    )
    happyRiders[noRiders] = 0 # steal the happiness from the no riders
    
    # update company's money with the profit from the rides
    comEarnings = (yesRides * driverCost)
    companyMoney = companyMoney + comEarnings
    
    # driver quits 
    driverQuits = rbinom( # randomly select
      n=drivers, # for each driver
      size=1,
      prob=driverQuitRate # if (s)he quits
    )
    driverQuits[which(is.na(driverQuits))] = 0  # because currently the quit rate can be negative for small enough cost
    quittedDrivers = sum(driverQuits) # their number
    
    # rider quits 
    riderQuits = rbinom( # randomly select
      n=riders, # for each rider
      size=1,
      prob=ifelse( # if (s)he quits
        happyRiders==1, # depending on their happiness
        happyRiderQuitRate, 
        sadRiderQuitRate
      )
    )
    quittedRiders = sum(riderQuits) # their number
    
    # update their numbers after they quit
    drivers = drivers - quittedDrivers
    riders = riders - quittedRiders
    # update the happyRiders
    happyRiders = happyRiders[which(riderQuits==0)]
    
    # end of month acquisition of new drivers and riders; must have a specific strategy (finally, it will be plain replacement of the lost customers)
    acqDrivers = quittedDrivers # this for testing
    acqRiders = quittedRiders # this for testing
    driverLosings = (acqDrivers * driverAC)
    riderLosings =  (acqRiders * riderAC)
    comLosings = driverLosings + riderLosings
    companyMoney = companyMoney - comLosings # update the company's money with the CAC
    comNet = comEarnings - comLosings
    happyRiders = c(happyRiders, rep(1, acqRiders))
    
    dataPre = rbind(
      dataPre,
      data.frame(
        rep,
        month,
        companyMoney,
        comMoneyPercInitCAC = companyMoney/initCAC*100,
        comEarnings,
        comLosings,
        driverLosings,
        riderLosings,
        comNet,
        comNetRel=comNet/initCAC*100,
        drivers,
        quittedDrivers,
        riders,
        quittedRiders,
        yesRate = yesRides / (yesRides+noRides) * 100
      )
    )
  }
}

## preparing the data for plotting
# total net revenue per run
dataPreAggr = aggregate(
  x=dataPre,
  comNetRel ~ rep,
  sum
)
meanTot = round(mean(dataPreAggr$comNetRel), 1)
seTot = round(sd(dataPreAggr$comNetRel) / sqrt(nrow(dataPreAggr)),1)
ciTot = round(
  qt(0.975,df=nrow(dataPreAggr)-1)*sd(dataPreAggr$comNetRel)/sqrt(nrow(dataPreAggr)),
  1
)

# monthly mean of net revenue
dataPreMean = aggregate(
  x=dataPre,
  comNetRel ~ month,
  mean
)

# the plot
dataPrePlot = ggplot(
  dataPre,
  aes(x=month,
      y=comNetRel
  )
) +
  geom_line(
    aes(group=rep),
    color="blue",
    alpha=0.2
  ) +
  geom_line(
    data=dataPreMean,
    aes(
      x=month,
      y=comNetRel
    ),
    color="magenta",
    linewidth=1
  ) +
  # geom_point(shape=1) +
  # scale_y_continuous(
  #   label=dollar_format()) +
  scale_x_continuous(
    breaks=monthsVec) +
  geom_label(
    data=data.frame(
      month=12,
      height=max(dataPre$comNetRel),
      text=paste0(
        meanTot,
        "\u00b1",
        ciTot,
        "%"
      )
    ),
    aes(x=month,
        y=height,
        label=text
    ),
    nudge_x=-0.9,
    nudge_y=-0.3,
    size=2,
    color="magenta"
  ) +
  labs(x="Month",
       y="Net revenue (% initial CAC)") +
  plotTheme2 +
  theme(panel.grid.minor=element_blank())
dataPrePlot

ggsave(
  filename="output/netPre.pdf",
  width=8,
  height=5,
  units="cm"
)

# validating the mathematical model
dataPreAggr = aggregate(
  x=dataPre,
  driverLosings ~ month,
  mean
)
ggplot(
  dataPre,
  aes(x=month,
      y=driverLosings
  )
) +
  geom_line(
    aes(group=rep),
    color="blue",
    alpha=0.2
  ) +
  geom_line(
    data=dataPreAggr,
    aes(
      x=month,
      y=sum(driverLosings)
    ),
    color="magenta",
    linewidth=1
  ) +
  # geom_point(shape=1) +
  # scale_y_continuous(
  #   label=dollar_format()) +
  scale_x_continuous(
    breaks=monthsVec) +
  labs(x="Month",
       y="Net revenue (% initial CAC)") +
  plotTheme2 +
  theme(panel.grid.minor=element_blank())



## the model ($3 charged, x rides per month, q driver quit rate, 93% answering) ####

# given the fixed numbers and ratio of drivers and riders, to have 93% answering of 16600 riders from 100 drivers, the number of rides of a driver must be:
round(16600*0.93/100)

# the answering increased by 55%, from 60% to 93%, and equivalently the driver quit rate will be decreased by 55%, from 20% to (q-20)/20*100=-55 => q = -55*20/100+20 => q=9%


monthsVec = 1:12 # the month indices
initAcqDrivers = 100 # will it be a variable later?
initAcqRiders = 16600 # will it be a variable later?
# riderCost = 25 # the cost of a ride for a rider
driverCost = 3 # the cost of a ride for a driver (paid to the company); will it be variable later?
driverRideRate = round(16600*0.93/100) # the monthly rate of rides; will it become variable later, as a function of driverCost?
driverQuitRate = 0.09 # the monthly rate of driver quit
happyRiderQuitRate = 0.1 # the monthly rate of a rider who has been served so far 
sadRiderQuitRate = 0.33 # the monthly rate of a rider who has not been served at least once in the past
repsCount = 100

set.seed(112)
dataExp = NULL
for (rep in 1:repsCount)
{
  print(rep)
  
  companyMoney = 0 # the budget of the company
  drivers = 0 # the initial number of drivers
  riders = 0 # the initial number of riders

  
  # at the end of the previous month, acquisitions are assumed to have occurred
  acqDrivers = initAcqDrivers # this for testing; 
  acqRiders = initAcqRiders # this for testing;
  initCAC = (acqDrivers * driverAC) + (acqRiders * riderAC)
  companyMoney = companyMoney - initCAC # update the company's money with the CAC
  happyRiders = rep(1, acqRiders)
  
  
  # the monthly loop
  for(month in monthsVec)
  {
    drivers = drivers + acqDrivers
    riders = riders + acqRiders
    
    # assuming that each acquired rider will call once per month
    
    # calls during the month
    yesRides = min( # the number of answered calls
      riders, # if there are enough drivers, all riders are answered
      drivers * driverRideRate
    )
    noRides = riders - yesRides
    noRiders = sample( # select randomly
      x=1:riders, # which of the riders
      size=noRides, # were not answered
      replace=FALSE
    )
    happyRiders[noRiders] = 0 # steal the happiness from the no riders
    
    # update company's money with the profit from the rides
    comEarnings = (yesRides * driverCost)
    companyMoney = companyMoney + comEarnings
    
    # driver quits 
    driverQuits = rbinom( # randomly select
      n=drivers, # for each driver
      size=1,
      prob=driverQuitRate # if (s)he quits
    )
    driverQuits[which(is.na(driverQuits))] = 0  # because currently the quit rate can be negative for small enough cost
    quittedDrivers = sum(driverQuits) # their number
    
    # rider quits 
    riderQuits = rbinom( # randomly select
      n=riders, # for each rider
      size=1,
      prob=ifelse( # if (s)he quits
        happyRiders==1, # depending on their happiness
        happyRiderQuitRate, 
        sadRiderQuitRate
      )
    )
    quittedRiders = sum(riderQuits) # their number
    
    # update their numbers after they quit
    drivers = drivers - quittedDrivers
    riders = riders - quittedRiders
    # update the happyRiders
    happyRiders = happyRiders[which(riderQuits==0)]
    
    # end of month acquisition of new drivers and riders; must have a specific strategy (finally, it will be plain replacement of the lost customers)
    acqDrivers = quittedDrivers # this for testing
    acqRiders = quittedRiders # this for testing
    driverLosings = (acqDrivers * driverAC)
    riderLosings = acqRiders * riderAC
    comLosings = driverLosings + riderLosings
    companyMoney = companyMoney - comLosings # update the company's money with the CAC
    comNet = comEarnings - comLosings
    happyRiders = c(happyRiders, rep(1, acqRiders))
    
    dataExp = rbind(
      dataExp,
      data.frame(
        rep,
        month,
        companyMoney,
        comMoneyPercInitCAC = companyMoney/initCAC*100,
        comEarnings,
        comLosings,
        driverLosings,
        riderLosings,
        comNet,
        comNetRel=comNet/initCAC*100,
        drivers,
        quittedDrivers,
        riders,
        quittedRiders,
        yesRate = yesRides / (yesRides+noRides) * 100
      )
    )
  }
}

## preparing data for plotting; see previous section for the $6
dataExpAggr = aggregate(
  x=dataExp,
  comNetRel ~ rep,
  sum
)
meanTot = round(mean(dataExpAggr$comNetRel), 1)
seTot = round(sd(dataExpAggr$comNetRel) / sqrt(nrow(dataExpAggr)),1)
ciTot = round(
  qt(0.975,df=nrow(dataExpAggr)-1)*sd(dataExpAggr$comNetRel)/sqrt(nrow(dataExpAggr)),
  1
)

dataExpMean = aggregate(
  x=dataExp,
  comNetRel ~ month,
  mean
)

dataExpPlot = ggplot(
  dataExp,
  aes(x=month,
      y=comNetRel
  )
) +
  geom_line(
    aes(group=rep),
    color="blue",
    alpha=0.2
  ) +
  geom_line(
    data=dataExpMean,
    aes(
      x=month,
      y=comNetRel
    ),
    color="magenta",
    linewidth=1
  ) +
  # geom_point(shape=1) +
  # scale_y_continuous(
  #   label=dollar_format()) +
  scale_x_continuous(
    breaks=monthsVec) +
  geom_label(
    data=data.frame(
      month=12,
      height=max(dataExp$comNetRel),
      text=paste0(
        meanTot,
        "\u00b1",
        ciTot,
        "%"
      )
    ),
    aes(x=month,
        y=height,
        label=text
    ),
    nudge_x=-0.8,
    nudge_y=-0.2,
    size=2,
    color="magenta"
  ) +
  labs(x="Month",
     y="Net revenue (% initial CAC)") +
  plotTheme2 +
  theme(panel.grid.minor=element_blank())
dataExpPlot

ggsave(
  filename="output/netExp.pdf",
  width=8,
  height=5,
  units="cm"
)

## multi-panel of $6 and $3 cases (Figure 2 in main text)
dataPreExpPlot = ggarrange(
  dataPrePlot,
  dataExpPlot,
  labels=paste0("(", letters[1:2], ")"),
  label.args = list(hjust=-3.8, vjust=2, gp=gpar(fontsize=8))
)
dataPreExpPlot

ggsave(
  plot=dataPreExpPlot,
  filename="output/netPreExp.pdf",
  width=8,
  height=10,
  units="cm"
)

# checking the mathematical model
dataExpAggr = aggregate(
  x=dataExp,
  riderLosings ~ month,
  mean
)
ggplot(
  dataExp,
  aes(x=month,
      y=riderLosings
  )
) +
  geom_line(
    aes(group=rep),
    color="blue",
    alpha=0.2
  ) +
  geom_line(
    data=dataExpAggr,
    aes(
      x=month,
      y=sum(riderLosings)
    ),
    color="magenta",
    linewidth=1
  ) +
  # geom_point(shape=1) +
  # scale_y_continuous(
  #   label=dollar_format()) +
  scale_x_continuous(
    breaks=monthsVec) +
labs(x="Month",
     y="Net revenue (% initial CAC)") +
  plotTheme2 +
  theme(panel.grid.minor=element_blank())

sum(dataExpAggr$riderLosings)

## relation between monthly number of rides versus driver charge cost per ride and driver quit rate ####

# the driver rides model
rideChargeRelDF = data.frame(
  charged=c(3, 6),
  rides=c(154, 100)
)
rideChargeModel = lm(
  rides ~ charged,
  data=rideChargeRelDF
)

# the driver quit rate model
driverQuitRelDF = data.frame(
  charged=c(3, 6),
  quit=c(0.09, 0.2)
)
driverQuitModel = lm(
  quit ~ charged,
  data=driverQuitRelDF
)

# put together for plotting
relDF = cbind(
  rideChargeRelDF,
  quit=driverQuitRelDF$quit
)

# plotting the two relations (Figure 1 in main text)
chargeMin = 1
chargeMax = 10
secondYscaling = 10
quitScaling = 1000
ggplot(
  relDF,
  aes(
    x=charged
  )
) +
  geom_abline(
    slope=rideChargeModel$coefficients[2],
    intercept=rideChargeModel$coefficients[1],
    color="magenta"
  ) +
  geom_abline(
    slope=driverQuitModel$coefficients[2]*quitScaling,
    intercept=driverQuitModel$coefficients[1]*quitScaling,
    color="gray"
  ) +
  geom_point(
    aes(y=rides),
    shape=1
  ) +
  geom_point(
    aes(y=quit*quitScaling),
    shape=4
  ) +
  scale_x_continuous(
    limits=c(chargeMin, chargeMax),
    breaks=chargeMin:chargeMax, #seq(1, 10, 3)
  ) +
  scale_y_continuous(
    limits=c(20, 300),
    name="Driver monthly number of rides",
    sec.axis=sec_axis(~./secondYscaling, name="Driver monthly quit rate (%)")
  ) +
  labs(
    x="Driver cost per ride (\u0024)"
  ) +
  plotTheme2 +
  theme(
    axis.title.y = element_text(color="magenta"),
    axis.title.y.right = element_text(color="gray")
  )


ggsave(
  filename="output/relModels.pdf",
  width=8,
  height=6,
  units="cm"
)

## the model (variable driver cost at the highest accuracy, variable rides per month, variable answering, what's the average money in the 12th month?) ####

# load directly if already run; else, run it with the following code
load("input/dataRelFinest.rda")

# the driver rides model
rideChargeRelDF = data.frame(
  charged=c(3, 6),
  rides=c(154, 100)
)
rideChargeModel = lm(
  rides ~ charged,
  data=rideChargeRelDF
)

# the driver quit rate model
driverQuitRelDF = data.frame(
  charged=c(3, 6),
  quit=c(0.09, 0.2)
)
driverQuitModel = lm(
  quit ~ charged,
  data=driverQuitRelDF
)

set.seed(112)
costValues=seq(2.3,2.5,0.01)
dataRelFinest = NULL
for (driverCost in costValues)
{
  # riderCost = 25 # the cost of a ride for a rider
  driverRideRate = round(predict(rideChargeModel, data.frame(charged=driverCost))) # the monthly rate of rides
  driverQuitRate = predict(driverQuitModel, data.frame(charged=driverCost)) # the monthly rate of driver quit
  happyRiderQuitRate = 0.1 # the monthly rate of a rider who has been served so far 
  sadRiderQuitRate = 0.33 # the monthly rate of a rider who has not been served at least once in the past
  repsCount = 100
  
  for (rep in 1:repsCount)
  {
    print(c(driverCost, rep))
    
    companyMoney = 0 # the budget of the company
    drivers = 0 # the initial number of drivers
    riders = 0 # the initial number of riders
    monthsVec = 1:12
    initAcqDrivers = 100 # will it be a variable later?
    initAcqRiders = 16600 # will it be a variable later?
    
    # at the end of the previous month, acquisitions are assumed to have occurred
    acqDrivers = initAcqDrivers # this for testing; 
    acqRiders = initAcqRiders # this for testing;
    initCAC = (acqDrivers * driverAC) + (acqRiders * riderAC)
    companyMoney = companyMoney - initCAC # update the company's money with the CAC
    happyRiders = rep(1, acqRiders)
    
    
    # the monthly loop
    for(month in monthsVec)
    {
      drivers = drivers + acqDrivers
      riders = riders + acqRiders
      
      # assuming that each acquired rider will call once per month
      
      # calls during the month
      yesRides = min( # the number of answered calls
        riders, # if there are enough drivers, all riders are answered
        drivers * driverRideRate
      )
      noRides = riders - yesRides
      noRiders = sample( # select randomly
        x=1:riders, # which of the riders
        size=noRides, # were not answered
        replace=FALSE
      )
      happyRiders[noRiders] = 0 # steal the happiness from the no riders
      
      # update company's money with the profit from the rides
      comEarnings = (yesRides * driverCost)
      companyMoney = companyMoney + comEarnings
      
      # driver quits 
      driverQuits = rbinom( # randomly select
        n=drivers, # for each driver
        size=1,
        prob=driverQuitRate # if (s)he quits
      )
      driverQuits[which(is.na(driverQuits))] = 0  # because currently the quit rate can be negative for small enough cost
      quittedDrivers = sum(driverQuits) # their number
      
      # rider quits 
      riderQuits = rbinom( # randomly select
        n=riders, # for each rider
        size=1,
        prob=ifelse( # if (s)he quits
          happyRiders==1, # depending on their happiness
          happyRiderQuitRate, 
          sadRiderQuitRate
        )
      )
      quittedRiders = sum(riderQuits) # their number
      
      # update their numbers after they quit
      drivers = drivers - quittedDrivers
      riders = riders - quittedRiders
      # update the happyRiders
      happyRiders = happyRiders[which(riderQuits==0)]
      
      # end of month acquisition of new drivers and riders; must have a specific strategy (finally, it will be plain replacement of the lost customers)
      acqDrivers = quittedDrivers # this for testing
      acqRiders = quittedRiders # this for testing
      comLosings = (acqDrivers * driverAC) + (acqRiders * riderAC)
      companyMoney = companyMoney - comLosings # update the company's money with the CAC
      comNet = comEarnings - comLosings
      happyRiders = c(happyRiders, rep(1, acqRiders))
      
      dataRelFinest = rbind(
        dataRelFinest,
        data.frame(
          driverCost,
          rep,
          month,
          companyMoney,
          comMoneyPercInitCAC = companyMoney/initCAC*100,
          comEarnings,
          comLosings,
          comNet,
          comNetRel=comNet/initCAC*100,
          drivers,
          quittedDrivers,
          riders,
          quittedRiders,
          yesRate = yesRides / (yesRides+noRides) * 100
        )
      )
    }
  }
}

# save to file, to load next time, instead of running again
save(dataRelFinest,
     file="input/dataRelFinest.rda")

## preparing the data for plotting
# total revenue per run and driver cost
dataRelFinestTotal = aggregate(
  x=dataRelFinest,
  comNetRel ~ rep + driverCost,
  sum
)

# mean total net revenue per driver cost
dataRelFinestTotalMean = aggregate(
  x=dataRelFinestTotal,
  comNetRel ~ driverCost,
  mean
)
# std. deviation
dataRelFinestTotalSd = aggregate(
  x=dataRelFinestTotal,
  comNetRel ~ driverCost,
  sd
)

## margins for std. deviation, std. error and conf. intervals
dataRelFinestAggr = dataRelFinestTotalMean
#
dataRelFinestAggr$sdev = dataRelFinestTotalSd$comNetRel #/ sqrt(repsCount)
dataRelFinestAggr$sdLow = dataRelFinestAggr$comNetRel - dataRelFinestAggr$sdev
dataRelFinestAggr$sdHigh = dataRelFinestAggr$comNetRel + dataRelFinestAggr$sdev
#
dataRelFinestAggr$sErr =  dataRelFinestAggr$sdev / sqrt(repsCount)
dataRelFinestAggr$sErrLow = dataRelFinestAggr$comNetRel -
  dataRelFinestAggr$sErr
dataRelFinestAggr$sErrHigh = dataRelFinestAggr$comNetRel +
  dataRelFinestAggr$sErr
#
dataRelFinestAggr$ci = qt(0.975,df=repsCount-1)*dataRelFinestAggr$sdev/sqrt(repsCount)
dataRelFinestAggr$ciLow = dataRelFinestAggr$comNetRel -
  dataRelFinestAggr$ci
dataRelFinestAggr$ciHigh = dataRelFinestAggr$comNetRel +
  dataRelFinestAggr$ci

## the model (variable driver cost at higher accuracy, variable rides per month, variable answering, what's the average money in the 12th month?) ####

# load if you have already run and saved the following code
load("input/dataRelFine.rda")

# the driver rides model
rideChargeRelDF = data.frame(
  charged=c(3, 6),
  rides=c(154, 100)
)
rideChargeModel = lm(
  rides ~ charged,
  data=rideChargeRelDF
)

# the driver quit rate model
driverQuitRelDF = data.frame(
  charged=c(3, 6),
  quit=c(0.09, 0.2)
)
driverQuitModel = lm(
  quit ~ charged,
  data=driverQuitRelDF
)

set.seed(112)
costValues=seq(2,4,0.1)
dataRelFine = NULL
for (driverCost in costValues)
{
  # riderCost = 25 # the cost of a ride for a rider
  driverRideRate = round(predict(rideChargeModel, data.frame(charged=driverCost)))
  driverQuitRate = predict(driverQuitModel, data.frame(charged=driverCost)) # the monthly rate of driver quit
  happyRiderQuitRate = 0.1 # the monthly rate of a rider who has been served so far 
  sadRiderQuitRate = 0.33 # the monthly rate of a rider who has not been served at least once in the past
  repsCount = 100
  
  for (rep in 1:repsCount)
  {
    print(c(driverCost, rep))
    
    companyMoney = 0 # the budget of the company
    drivers = 0 # the initial number of drivers
    riders = 0 # the initial number of riders
    monthsVec = 1:12
    initAcqDrivers = 100 # will it be a variable later?
    initAcqRiders = 16600 # will it be a variable later?
    
    # at the end of the previous month, acquisitions are assumed to have occurred
    acqDrivers = initAcqDrivers # this for testing; 
    acqRiders = initAcqRiders # this for testing;
    initCAC = (acqDrivers * driverAC) + (acqRiders * riderAC)
    companyMoney = companyMoney - initCAC # update the company's money with the CAC
    happyRiders = rep(1, acqRiders)
    
    
    # the monthly loop
    for(month in monthsVec)
    {
      drivers = drivers + acqDrivers
      riders = riders + acqRiders
      
      # assuming that each acquired rider will call once per month
      
      # calls during the month
      yesRides = min( # the number of answered calls
        riders, # if there are enough drivers, all riders are answered
        drivers * driverRideRate
      )
      noRides = riders - yesRides
      noRiders = sample( # select randomly
        x=1:riders, # which of the riders
        size=noRides, # were not answered
        replace=FALSE
      )
      happyRiders[noRiders] = 0 # steal the happiness from the no riders
      
      # update company's money with the profit from the rides
      comEarnings = (yesRides * driverCost)
      companyMoney = companyMoney + comEarnings
      
      # driver quits 
      driverQuits = rbinom( # randomly select
        n=drivers, # for each driver
        size=1,
        prob=driverQuitRate # if (s)he quits
      )
      driverQuits[which(is.na(driverQuits))] = 0  # because currently the quit rate can be negative for small enough cost
      quittedDrivers = sum(driverQuits) # their number
      
      # rider quits 
      riderQuits = rbinom( # randomly select
        n=riders, # for each rider
        size=1,
        prob=ifelse( # if (s)he quits
          happyRiders==1, # depending on their happiness
          happyRiderQuitRate, 
          sadRiderQuitRate
        )
      )
      quittedRiders = sum(riderQuits) # their number
      
      # update their numbers after they quit
      drivers = drivers - quittedDrivers
      riders = riders - quittedRiders
      # update the happyRiders
      happyRiders = happyRiders[which(riderQuits==0)]
      
      # end of month acquisition of new drivers and riders; must have a specific strategy (finally, it will be plain replacement of the lost customers)
      acqDrivers = quittedDrivers # this for testing
      acqRiders = quittedRiders # this for testing
      comLosings = (acqDrivers * driverAC) + (acqRiders * riderAC)
      companyMoney = companyMoney - comLosings # update the company's money with the CAC
      comNet = comEarnings - comLosings
      happyRiders = c(happyRiders, rep(1, acqRiders))
      
      dataRelFine = rbind(
        dataRelFine,
        data.frame(
          driverCost,
          rep,
          month,
          companyMoney,
          comMoneyPercInitCAC = companyMoney/initCAC*100,
          comEarnings,
          comLosings,
          comNet,
          comNetRel=comNet/initCAC*100,
          drivers,
          quittedDrivers,
          riders,
          quittedRiders,
          yesRate = yesRides / (yesRides+noRides) * 100
        )
      )
    }
  }
}

# save the data
save(dataRelFine,
     file="input/dataRelFine.rda")

## preparing the data (see comments for the highest accuracy in the section before)
dataRelFineTotal = aggregate(
  x=dataRelFine,
  comNetRel ~ rep + driverCost,
  sum
)

dataRelFineTotalMean = aggregate(
  x=dataRelFineTotal,
  comNetRel ~ driverCost,
  mean
)

dataRelFineTotalSd = aggregate(
  x=dataRelFineTotal,
  comNetRel ~ driverCost,
  sd
)

dataRelFineAggr = dataRelFineTotalMean
#
dataRelFineAggr$sdev = dataRelFineTotalSd$comNetRel #/ sqrt(repsCount)
dataRelFineAggr$sdLow = dataRelFineAggr$comNetRel -
  dataRelFineAggr$sdev
dataRelFineAggr$sdHigh = dataRelFineAggr$comNetRel +
  dataRelFineAggr$sdev
#
dataRelFineAggr$sErr =  dataRelFineAggr$sdev / sqrt(repsCount)
dataRelFineAggr$sErrLow = dataRelFineAggr$comNetRel -
  dataRelFineAggr$sErr
dataRelFineAggr$sErrHigh = dataRelFineAggr$comNetRel +
  dataRelFineAggr$sErr
#
dataRelFineAggr$ci = qt(0.975,df=repsCount-1)*dataRelFineAggr$sdev/sqrt(repsCount)
dataRelFineAggr$ciLow = dataRelFineAggr$comNetRel -
  dataRelFineAggr$ci
dataRelFineAggr$ciHigh = dataRelFineAggr$comNetRel +
  dataRelFineAggr$ci

# loading and preparing the data from the mathematical model (Mathematica)
totalNetMeanField = read.table(
  "/home/danis/Desktop/Amanda/Lyft/mathematica/totalNetPlotData.txt"
)
names(totalNetMeanField) = c("driverCost", "comNetRel")
interpol = data.frame(
  approx(
    totalNetMeanField$driverCost,
    totalNetMeanField$comNetRel,
    xout=c(2,4)
  )
)
names(interpol) = c("driverCost", "comNetRel")
totalNetMeanField = rbind(
  totalNetMeanField,
  interpol
)
totalNetMeanFieldFine = subset(
  totalNetMeanField,
  driverCost <= 4 & driverCost >=2
)

# plotting 
netVScostFiner = ggplot(
  dataRelFineAggr,
  aes(
    x=driverCost,
    y=comNetRel
  )
) +
  geom_vline(
    xintercept=dataRelFinestAggr$driverCost[which.max(dataRelFinestAggr$comNetRel)],
    linetype="dashed",
    color="magenta"#,
    # linewidth=0.2
  ) +
  geom_errorbar(
    aes(ymin=ciLow,
        ymax=ciHigh),
    color="blue",
    linewidth=0.1
  ) +
  geom_point(
    shape=21,
    fill="white",
    color="blue"#,
    # size=0.5
  ) +
  scale_x_continuous(
    breaks=seq(min(dataRelFineAggr$driverCost), max(dataRelFineAggr$driverCost), 0.4),
  ) +
  # scale_y_continuous(
  #   label=dollar_format()) +
  geom_label(
    data=data.frame(
      x=max(dataRelFineAggr$driverCost),
      y=max(dataRelFineAggr$comNetRel),
      text=paste0(
        round(max(dataRelFinestAggr$comNetRel), 1),
        "\u00b1",
        round(dataRelFinestAggr$ci[which.max(dataRelFinestAggr$comNetRel)],1),
        "%"
      )
    ),
    aes(x=x,
        y=y,
        label=text
    ),
    nudge_x=-0.17,
    nudge_y=1,
    size=2,
    color="magenta"
  ) +
  annotate(
    "text",
    x=dataRelFinestAggr$driverCost[which.max(dataRelFinestAggr$comNetRel)],
    y=min(dataRelFineAggr$comNetRel),
    label=as.character(dataRelFinestAggr$driverCost[which.max(dataRelFinestAggr$comNetRel)]),
    angle=90,
    hjust=0.5,
    vjust=-0.2,
    size=2.5,
    color="magenta"
  ) +
  geom_line(
    data=totalNetMeanFieldFine,
    aes(x=driverCost,
        comNetRel),
    color="black"#,
    # linewidth=0.2
  ) +
  coord_cartesian(
    xlim = c(
      min(dataRelFineAggr$driverCost), 
      max(dataRelFineAggr$driverCost)), 
    clip = "off") +
  labs(
    x="Driver cost per ride (\u0024)",
    y="Total net revenue (% initial CAC)"
  ) +
  plotTheme2 #+
  # theme(
  #   axis.title=element_blank(),
  #   axis.text=element_text(size=5)
  # )
netVScostFiner

# netVScostFineGrob = ggplotGrob(netVScostFine)

ggsave(
  filename="output/netVScostFiner.pdf",
  width=8,
  height=6,
  units="cm"
)




## the model (driver cost at the $1 accuracy, variable rides per month, variable driver quits, variable answering, what's the average money in the 12th month?) ####

# load if you have saved the data, instead of running again the following code
load("input/dataRel.rda")

# the driver rides model
rideChargeRelDF = data.frame(
  charged=c(3, 6),
  rides=c(154, 100)
)
rideChargeModel = lm(
  rides ~ charged,
  data=rideChargeRelDF
)

# the driver quit rate model
driverQuitRelDF = data.frame(
  charged=c(3, 6),
  quit=c(0.09, 0.2)
)
driverQuitModel = lm(
  quit ~ charged,
  data=driverQuitRelDF
)

set.seed(112)
costValues=1:10
dataRel = NULL
for (driverCost in costValues)
{
  # riderCost = 25 # the cost of a ride for a rider
  driverRideRate = round(predict(rideChargeModel, data.frame(charged=driverCost))) # the monthly rate of rides
  driverQuitRate = predict(driverQuitModel, data.frame(charged=driverCost)) # the monthly rate of driver quit
  happyRiderQuitRate = 0.1 # the monthly rate of a rider who has been served so far 
  sadRiderQuitRate = 0.33 # the monthly rate of a rider who has not been served at least once in the past
  repsCount = 100
  
  for (rep in 1:repsCount)
  {
    print(c(driverCost, rep))
    
    companyMoney = 0 # the budget of the company
    drivers = 0 # the initial number of drivers
    riders = 0 # the initial number of riders
    monthsVec = 1:12
    initAcqDrivers = 100 # will it be a variable later?
    initAcqRiders = 16600 # will it be a variable later?
    
    # at the end of the previous month, acquisitions are assumed to have occurred
    acqDrivers = initAcqDrivers # this for testing; 
    acqRiders = initAcqRiders # this for testing;
    initCAC = (acqDrivers * driverAC) + (acqRiders * riderAC)
    companyMoney = companyMoney - initCAC # update the company's money with the CAC
    happyRiders = rep(1, acqRiders)
    
    
    # the monthly loop
    for(month in monthsVec)
    {
      drivers = drivers + acqDrivers
      riders = riders + acqRiders
      
      # assuming that each acquired rider will call once per month
      
      # calls during the month
      yesRides = min( # the number of answered calls
        riders, # if there are enough drivers, all riders are answered
        drivers * driverRideRate
      )
      noRides = riders - yesRides
      noRiders = sample( # select randomly
        x=1:riders, # which of the riders
        size=noRides, # were not answered
        replace=FALSE
      )
      happyRiders[noRiders] = 0 # steal the happiness from the no riders
      
      # update company's money with the profit from the rides
      comEarnings = (yesRides * driverCost)
      companyMoney = companyMoney + comEarnings
      
      # driver quits 
      driverQuits = rbinom( # randomly select
        n=drivers, # for each driver
        size=1,
        prob=driverQuitRate # if (s)he quits
      )
      driverQuits[which(is.na(driverQuits))] = 0  # because currently the quit rate can be negative for small enough cost
      quittedDrivers = sum(driverQuits) # their number
      
      # rider quits 
      riderQuits = rbinom( # randomly select
        n=riders, # for each rider
        size=1,
        prob=ifelse( # if (s)he quits
          happyRiders==1, # depending on their happiness
          happyRiderQuitRate, 
          sadRiderQuitRate
        )
      )
      quittedRiders = sum(riderQuits) # their number
      
      # update their numbers after they quit
      drivers = drivers - quittedDrivers
      riders = riders - quittedRiders
      # update the happyRiders
      happyRiders = happyRiders[which(riderQuits==0)]
      
      # end of month acquisition of new drivers and riders; must have a specific strategy (finally, it will be plain replacement of the lost customers)
      acqDrivers = quittedDrivers # this for testing
      acqRiders = quittedRiders # this for testing
      comLosings = (acqDrivers * driverAC) + (acqRiders * riderAC)
      companyMoney = companyMoney - comLosings # update the company's money with the CAC
      comNet = comEarnings - comLosings
      happyRiders = c(happyRiders, rep(1, acqRiders))
      
      dataRel = rbind(
        dataRel,
        data.frame(
          driverCost,
          rep,
          month,
          companyMoney,
          comMoneyPercInitCAC = companyMoney/initCAC*100,
          comEarnings,
          comLosings,
          comNet,
          comNetRel=comNet/initCAC*100,
          drivers,
          quittedDrivers,
          riders,
          quittedRiders,
          yesRate = yesRides / (yesRides+noRides) * 100
        )
      )
    }
  }
}

# saving the data, to directly load them next time
save(dataRel,
     file="input/dataRel.rda")

# data preparation for plotting (see section for highest accuracy)
dataRelTotal = aggregate(
  x=dataRel,
  comNetRel ~ rep + driverCost,
  sum
)

dataRelTotalMean = aggregate(
  x=dataRelTotal,
  comNetRel ~ driverCost,
  mean
)

dataRelTotalSd = aggregate(
  x=dataRelTotal,
  comNetRel ~ driverCost,
  sd
)

dataRelAggr = dataRelTotalMean
#
dataRelAggr$sdev = dataRelTotalSd$comNetRel #/ sqrt(repsCount)
dataRelAggr$sdLow = dataRelAggr$comNetRel -
  dataRelAggr$sdev
dataRelAggr$sdHigh = dataRelAggr$comNetRel +
  dataRelAggr$sdev
#
dataRelAggr$sErr =  dataRelAggr$sdev / sqrt(repsCount)
dataRelAggr$sErrLow = dataRelAggr$comNetRel -
  dataRelAggr$sErr
dataRelAggr$sErrHigh = dataRelAggr$comNetRel +
  dataRelAggr$sErr
#
dataRelAggr$ci = qt(0.975,df=repsCount-1)*dataRelAggr$sdev/sqrt(repsCount)
dataRelAggr$ciLow = dataRelAggr$comNetRel -
  dataRelAggr$ci
dataRelAggr$ciHigh = dataRelAggr$comNetRel +
  dataRelAggr$ci

# load and prepare the data from the mathematical model (Mathematica)
totalNetMeanField = read.table(
  "/home/danis/Desktop/Amanda/Lyft/mathematica/totalNetPlotData.txt"
)
names(totalNetMeanField) = c("driverCost", "comNetRel")

netVScostRough = ggplot(
  dataRelAggr,
  aes(
    x=driverCost,
    y=comNetRel
  )
) +
  geom_vline(
    xintercept=dataRelFineAggr$driverCost[which.max(dataRelFineAggr$comNetRel)],
    linetype="dashed",
    color="magenta"
  ) +
  geom_errorbar(
    aes(ymin=ciLow,
        ymax=ciHigh),
    color="blue",
    linewidth=0.1
  ) +
  geom_point(
    shape=21,
    fill="white",
    color="blue"
  ) +
  geom_label(
    data=data.frame(
      x=max(dataRelAggr$driverCost),
      y=max(dataRelAggr$comNetRel),
      text=paste0(
        round(max(dataRelFineAggr$comNetRel), 1),
        "\u00b1",
        round(dataRelFineAggr$ci[which.max(dataRelFineAggr$comNetRel)], 1),
        "%"
      )
    ),
    aes(x=x,
        y=y,
        label=text
    ),
    nudge_x=-0.65,
    nudge_y=10,
    size=2,
    color="magenta"
  ) +
  scale_x_continuous(
    breaks=seq(min(dataRelAggr$driverCost), max(dataRelAggr$driverCost), 1),
  ) +
  # scale_y_continuous(
  #   label=dollar_format()) +
  annotate(
    "text",
    x=dataRelFineAggr$driverCost[which.max(dataRelFineAggr$comNetRel)],
    y=min(dataRelAggr$comNetRel),
    label=as.character(dataRelFineAggr$driverCost[which.max(dataRelFineAggr$comNetRel)]),
    angle=90,
    hjust=0.6,
    vjust=-0.2,
    size=2.5,
    color="magenta"
  ) +
  geom_line(
    data=totalNetMeanField,
    aes(x=driverCost,
        comNetRel),
    color="black"
  ) +
  coord_cartesian(xlim = c(min(dataRelAggr$driverCost), max(dataRelAggr$driverCost)), clip = "off") +
  labs(
    x="Driver cost per ride (\u0024)",
    y="Total net revenue (% initial CAC)"
  ) +
  plotTheme2
netVScostRough

ggsave(
  filename="output/netVScostRough.pdf",
  width=8,
  height=6,
  units="cm"
)



## multi-panel (Figure 3 in main text)
netVScost = ggarrange(
  netVScostRough,
  netVScostFiner,
labels=paste0("(", letters[1:2], ")"),
label.args = list(hjust=-4.1, vjust=2, gp=gpar(fontsize=8))
)
netVScost

ggsave(
  plot=netVScost,
  filename="output/netVScost.pdf",
  width=8,
  height=12,
  units="cm"
)


## the model with the optimum values ####

dataRelFinestAggr$driverCost[which.max(dataRelFinestAggr$comNetRel)]
# $2.33

# the driver rides model
rideChargeRelDF = data.frame(
  charged=c(3, 6),
  rides=c(154, 100)
)
rideChargeModel = lm(
  rides ~ charged,
  data=rideChargeRelDF
)

# the driver quit rate model
driverQuitRelDF = data.frame(
  charged=c(3, 6),
  quit=c(0.09, 0.2)
)
driverQuitModel = lm(
  quit ~ charged,
  data=driverQuitRelDF
)


monthsVec = 1:36
initAcqDrivers = 100 # will it be a variable later?
initAcqRiders = 16600 # will it be a variable later?
# riderCost = 25 # the cost of a ride for a rider
driverCost = 2.33 # dataRelFinestAggr$driverCost[which.max(dataRelFinestAggr$comNetRel)] # the cost of a ride for a driver (paid to the company)
driverRideRate = round(predict(rideChargeModel, data.frame(charged=driverCost))) # the monthly rate of rides
driverQuitRate = predict(driverQuitModel, data.frame(charged=driverCost)) # the monthly rate of driver quit
happyRiderQuitRate = 0.1 # the monthly rate of a rider who has been served so far 
sadRiderQuitRate = 0.33 # the monthly rate of a rider who has not been served at least once in the past
repsCount = 100

set.seed(112)
dataFin = NULL
for (rep in 1:repsCount)
{
  print(rep)
  
  companyMoney = 0 # the budget of the company
  drivers = 0 # the initial number of drivers
  riders = 0 # the initial number of riders
  
  
  # at the end of the previous month, acquisitions are assumed to have occurred
  acqDrivers = initAcqDrivers # this for testing; 
  acqRiders = initAcqRiders # this for testing;
  initCAC = (acqDrivers * driverAC) + (acqRiders * riderAC)
  companyMoney = companyMoney - initCAC # update the company's money with the CAC
  happyRiders = rep(1, acqRiders)
  
  
  # the monthly loop
  for(month in monthsVec)
  {
    drivers = drivers + acqDrivers
    riders = riders + acqRiders
    
    # assuming that each acquired rider will call once per month
    
    # calls during the month
    yesRides = min( # the number of answered calls
      riders, # if there are enough drivers, all riders are answered
      drivers * driverRideRate
    )
    noRides = riders - yesRides
    noRiders = sample( # select randomly
      x=1:riders, # which of the riders
      size=noRides, # were not answered
      replace=FALSE
    )
    happyRiders[noRiders] = 0 # steal the happiness from the no riders
    
    # update company's money with the profit from the rides
    comEarnings = (yesRides * driverCost)
    companyMoney = companyMoney + comEarnings
    
    # driver quits 
    driverQuits = rbinom( # randomly select
      n=drivers, # for each driver
      size=1,
      prob=driverQuitRate # if (s)he quits
    )
    driverQuits[which(is.na(driverQuits))] = 0  # because currently the quit rate can be negative for small enough cost
    quittedDrivers = sum(driverQuits) # their number
    
    # rider quits 
    riderQuits = rbinom( # randomly select
      n=riders, # for each rider
      size=1,
      prob=ifelse( # if (s)he quits
        happyRiders==1, # depending on their happiness
        happyRiderQuitRate, 
        sadRiderQuitRate
      )
    )
    quittedRiders = sum(riderQuits) # their number
    
    # update their numbers after they quit
    drivers = drivers - quittedDrivers
    riders = riders - quittedRiders
    # update the happyRiders
    happyRiders = happyRiders[which(riderQuits==0)]
    
    # end of month acquisition of new drivers and riders; must have a specific strategy (finally, it will be plain replacement of the lost customers)
    acqDrivers = quittedDrivers # this for testing
    acqRiders = quittedRiders # this for testing
    driverLosings = acqDrivers * driverAC
    riderLosings = acqRiders * riderAC
    comLosings = driverLosings + riderLosings
    companyMoney = companyMoney - comLosings # update the company's money with the CAC
    comNet = comEarnings - comLosings
    happyRiders = c(happyRiders, rep(1, acqRiders))
    
    dataFin = rbind(
      dataFin,
      data.frame(
        rep,
        month,
        companyMoney,
        comMoneyPercInitCAC = companyMoney/initCAC*100,
        comEarnings,
        comLosings,
        driverLosings,
        riderLosings,
        comNet,
        comNetRel=comNet/initCAC*100,
        drivers,
        quittedDrivers,
        riders,
        quittedRiders,
        yesRate = yesRides / (yesRides+noRides) * 100
      )
    )
  }
}

# data preparation
dataFinAggr = aggregate(
  x=dataFin,
  comNetRel ~ rep,
  sum
)
meanTot = round(mean(dataFinAggr$comNetRel), 1)
seTot = round(sd(dataFinAggr$comNetRel) / sqrt(nrow(dataFinAggr)),1)
ciTot = qt(0.975,df=nrow(dataFinAggr)-1)*sd(dataFinAggr$comNetRel)/sqrt(nrow(dataFinAggr))

dataFinMean = aggregate(
  x=dataFin,
  comNetRel ~ month,
  mean
)

# quantities from the mathematical model
mathModelNet = (16600*2.33 - 100 * 500 * driverQuitRate - 16600 * 15 * 0.1) / initCAC *100
mathModelNet # the monthly net revenue

dataFinAggrMean = aggregate(
  x=dataFin,
  comNetRel ~ rep,
  mean
)
dataFinAggrSd = aggregate(
  x=dataFin,
  comNetRel ~ rep,
  sd
)
mean(dataFinAggrMean$comNetRel) # the monthly mean net revenue
qt(0.975,df=nrow(dataFinAggrSd)-1)*sd(dataFinAggrSd$comNetRel)/sqrt(nrow(dataFinAggrSd)) # and its confidence interval

# plotting (Figure 5 in main text)
ggplot(
  dataFin,
  aes(x=month,
      y=comNetRel
  )
) +
  geom_line(
    aes(group=rep),
    color="blue",
    alpha=0.2
  ) +
  geom_line(
    data=dataFinMean,
    aes(
      x=month,
      y=comNetRel
    ),
    color="magenta",
    linewidth=1
  ) +
  # geom_point(shape=1) +
  # scale_y_continuous(
  #   label=dollar_format()) +
  scale_x_continuous(
    breaks=seq(1, 36, 5)
  ) +
  geom_label(
    data=data.frame(
      month=36,
      height=max(dataFin$comNetRel),
      text=paste0(
        meanTot,
        "\u00b1",
        round(ciTot,1),
        "%"
      )
    ),
    aes(x=month,
        y=height,
        label=text
    ),
    nudge_x=-2,
    nudge_y=-0.1,
    size=2,
    color="magenta"
  ) +
  geom_segment(
    aes(
      x=1, xend=36,
      y=mathModelNet, # mean(dataFinAggr$comNetRel)
      yend=mathModelNet, # mean(dataFinAggr$comNetRel)
    ),
    linetype="solid"
  ) +
  geom_segment(
    aes(
      x=round(100/mean(dataFinMean$comNetRel)), xend=round(100/mean(dataFinMean$comNetRel)),
      y=-Inf, yend=mean(dataFinMean$comNetRel)
    ),
    linetype="dotted"
  ) +
  annotate(
    "text",
    x=round(100/mean(dataFinMean$comNetRel)),
    y=min(dataFin$comNetRel),
    label=as.character(round(100/mean(dataFinMean$comNetRel))),
    angle=90,
    hjust=0.5,
    vjust=-0.2,
    size=2.5,
    color="black"
  ) +
  labs(x="Month",
       y="Net revenue (% initial CAC)") +
  plotTheme2 

ggsave(
  filename="output/netFin.pdf",
  width=8,
  height=5,
  units="cm"
)

# the money made every month for the main text, just in case....
# > mean(dataFinAggr$comNetRel) /100 * initCAC
# [1] 10466.05



# checking the mathematical model
dataFinAggr = aggregate(
  x=dataFin,
  riderLosings ~ month,
  mean
)
ggplot(
  dataFin,
  aes(x=month,
      y=riderLosings
  )
) +
  geom_line(
    aes(group=rep),
    color="blue",
    alpha=0.2
  ) +
  geom_line(
    data=dataFinAggr,
    aes(
      x=month,
      y=sum(riderLosings)
    ),
    color="magenta",
    linewidth=1
  ) +
  scale_x_continuous(
    breaks=monthsVec) +
labs(x="Month",
     y="Net revenue (% initial CAC)") +
  plotTheme2 +
  theme(panel.grid.minor=element_blank())

sum(dataFinAggr$riderLosings)

## the mathematical model for different slopes of driver quit rate ####

# the following is loading the data from the mathematical model produced in Mathematica

zetaVals = c(0.03667)
for(zetaValID in 1:12)
{
  zetaVal = zetaVals[length(zetaVals)] * 1.2
  zetaVals = c(zetaVals, zetaVal)
}

etaVals = 0.2 - zetaVals * 6 # based on the empirical observation

c_q0 = -etaVals/zetaVals

totalNetVarDF = NULL
for(val in 1:length(zetaVals))
{
  totalNetVar = read.table(
    file=paste0(
      "/home/danis/Desktop/Amanda/Lyft/mathematica/totalNetVarPlotData",
      val,
      ".txt"
    )
  )
  names(totalNetVar) = c("driverCost", "totalNet")
  totalNetVar$driverQuitSlope = zetaVals[val]*100
  
  totalNetVarDF = rbind(
    totalNetVarDF,
    totalNetVar)
}

## plotting (Figure 6 in main text)
myBreaks = c(0.04, 0.08, 0.16, 0.32)*100
netVScostVarDriverQuit = ggplot(
  totalNetVarDF,
  aes(
    x=driverCost,
    y=totalNet,
    group=driverQuitSlope
  )
) +
  geom_vline(
    xintercept=2.33,
    linetype="solid",
    color="magenta"
  ) +
  geom_vline(
    xintercept=c_q0,
    color="black",
    linewidth=0.2,
    linetype="dashed"
  ) +
  annotate(
    "text",
    x=2.33,
    y=-75,
    label="2.33",
    angle=90,
    hjust=0.7,
    vjust=-0.2,
    size=2.5,
    color="magenta"
  ) +
  geom_line(
    aes(color=driverQuitSlope),
    linewidth=0.4
  ) +
  scale_x_continuous(
    limits=c(1, 6.5),
    breaks=1:10) +
  scale_y_continuous(
    limits=c(-80, 60),
    breaks=seq(-80, 60, 40)) +
  scale_color_gradient(
    trans="log",
    low="black",
    high="khaki",
    breaks=myBreaks,
    labels=myBreaks
  ) +
  labs(
    x="Driver cost per ride (\u0024)",
    y="Total net revenue (% initial CAC)",
    color="Driver\nquit\nrate's\nslope\n(%)"
  ) +
  plotTheme2
netVScostVarDriverQuit
  
ggsave(
  filename="output/netVScostVarDriverQuit.pdf",
  width=8,
  height=6,
  units="cm"
)

## optimum driver cost as a function of riders:drivers ratio ####

# c_A = (R/D - epsilon)/delta
# delta*6 + epsilon = 100 => epsilon = 100 - delta*6 # since the only sure point is the $6--100 rides

# the driver rides model
rideChargeRelDF = data.frame(
  charged=c(3, 6),
  rides=c(154, 100)
)
rideChargeModel = lm(
  rides ~ charged,
  data=rideChargeRelDF
)

# solving Eq. 5 for different ratios R/D

ratioVec = seq(1, 200, 1)
varRatioOnlyDF = NULL
epsilon = rideChargeModel$coefficients[1]
delta = rideChargeModel$coefficients[2]
for(ratio in ratioVec)
{
  costA = (ratio - epsilon) / delta
  
  varRatioOnlyDF = rbind(
    varRatioOnlyDF,
    data.frame(
      ratio,
      costA
    )
  )
}

summary(varRatioOnlyDF)

# Figure 7 in main text
ggplot(
  varRatioOnlyDF,
  aes(
    x=ratio,
    y=costA
  )
) +
  geom_line(
    color="magenta"
  ) +
  annotate(
    "point",
    x=166,
    y=2.33,
    color="black",
    shape=1
  ) +
  # scale_x_continuous(
  #   breaks=c(1, 40, 90, 140, 190)
  # ) +
  labs(
    x="Ratio of riders:drivers",
    y="Optimum driver cost\nper ride (\u0024)"
  ) +
  plotTheme2

ggsave(
  filename="output/varRatioOnly.pdf",
  width=8,
  height=4,
  units="cm"
)

## fixed pricing scheme simulation with variable riders:drivers ratio, initial 166:1, fixed number of replenished, driver cost fixed at $2.33 ####

# the driver rides model
rideChargeRelDF = data.frame(
  charged=c(3, 6),
  rides=c(154, 100)
)
rideChargeModel = lm(
  rides ~ charged,
  data=rideChargeRelDF
)

# the driver quit rate model
driverQuitRelDF = data.frame(
  charged=c(3, 6),
  quit=c(0.09, 0.2)
)
driverQuitModel = lm(
  quit ~ charged,
  data=driverQuitRelDF
)

driverCost = 2.33
# riderCost = 25 # the cost of a ride for a rider
driverRideRate = as.numeric(round(predict(rideChargeModel, data.frame(charged=driverCost)))) # the monthly rate of rides
driverQuitRate = as.numeric(predict(driverQuitModel, data.frame(charged=driverCost))) # the monthly rate of driver quit
happyRiderQuitRate = 0.1 # the monthly rate of a rider who has been served so far 
sadRiderQuitRate = 0.33 # the monthly rate of a rider who has not been served at least once in the past
initAcqDrivers = 100 # will it be a variable later?
initAcqRiders = 16600 # will it be a variable later?
replDrivers = round(driverQuitRate * initAcqDrivers)
replRiders = round(happyRiderQuitRate * initAcqRiders)
repsCount = 500
dataVarRatioFixedCost = NULL
for (rep in 1:repsCount)
{
  print(rep)
  
  companyMoney = 0 # the budget of the company
  drivers = 0 # the initial number of drivers
  riders = 0 # the initial number of riders
  monthsVec = 1:12

  # at the end of the previous month, acquisitions are assumed to have occurred
  acqDrivers = initAcqDrivers # this for testing; 
  acqRiders = initAcqRiders # this for testing;
  initCAC = (acqDrivers * driverAC) + (acqRiders * riderAC)
  companyMoney = companyMoney - initCAC # update the company's money with the CAC
  happyRiders = rep(1, acqRiders)
  
  # the monthly loop
  for(month in monthsVec)
  {
    drivers = drivers + acqDrivers
    riders = riders + acqRiders
    
    # assuming that each acquired rider will call once per month
    
    set.seed(rep + month)
    
    # calls during the month
    yesRides = min( # the number of answered calls
      riders, # if there are enough drivers, all riders are answered
      drivers * driverRideRate
    )
    noRides = riders - yesRides
    noRiders = sample( # select randomly
      x=1:riders, # which of the riders
      size=noRides, # were not answered
      replace=FALSE
    )
    happyRiders[noRiders] = 0 # steal the happiness from the no riders
    
    # update company's money with the profit from the rides
    comEarnings = (yesRides * driverCost)
    companyMoney = companyMoney + comEarnings
    
    # driver quits 
    # replDrivers = round(drivers*driverQuitRate)
    # quittedDrivers = rpois(n=1, lambda=replDrivers)
    driverQuits = rbinom( # randomly select
      n=drivers, # for each driver
      size=1,
      prob=driverQuitRate # if (s)he quits
    )
    driverQuits[which(is.na(driverQuits))] = 0  # because currently the quit rate can be negative for small enough cost
    quittedDrivers = sum(driverQuits) # their number
    
   riderQuits = rbinom( # randomly select
    n=riders, # for each rider
    size=1,
    prob=ifelse( # if (s)he quits
      happyRiders==1, # depending on their happiness
      happyRiderQuitRate,
      sadRiderQuitRate
      )
    )
    quittedRiders = sum(riderQuits) # their number
    
    # update their numbers after they quit
    drivers = drivers - quittedDrivers
    riders = riders - quittedRiders
    # update the happyRiders
    happyRiders = happyRiders[which(riderQuits==0)]
    
    # end of month acquisition of new drivers and riders; must have a specific strategy (finally, it will be plain replacement of the lost customers)
    acqDrivers = rpois(n=1, lambda=quittedDrivers) #max(0, round(rnorm(n=1, mean=quittedDrivers, sd=2))) #rpois(n=1, lambda=quittedDrivers) #replDrivers
    acqRiders = rpois(n=1, lambda=quittedRiders) #max(0, round(rnorm(n=1, mean=quittedRiders, sd=150))) #rpois(n=1, lambda=quittedRiders) #replRiders
    comLosings = (acqDrivers * driverAC) + (acqRiders * riderAC)
    companyMoney = companyMoney - comLosings # update the company's money with the CAC
    comNet = comEarnings - comLosings
    happyRiders = c(happyRiders, rep(1, acqRiders))
    
    dataVarRatioFixedCost = rbind(
      dataVarRatioFixedCost,
      data.frame(
        rep,
        month,
        companyMoney,
        comMoneyPercInitCAC = companyMoney/initCAC*100,
        comEarnings,
        comLosings,
        comNet,
        comNetRel=comNet/initCAC*100,
        drivers,
        quittedDrivers,
        riders,
        quittedRiders,
        yesRate = yesRides / (yesRides+noRides) * 100
      )
    )
  }
}

# preparing data for plotting
dataVarRatioFixedCostAggr = aggregate(
  x=dataVarRatioFixedCost,
  comNetRel ~ rep,
  sum
)
meanTotFix = round(mean(dataVarRatioFixedCostAggr$comNetRel), 1)
seTot = round(sd(dataVarRatioFixedCostAggr$comNetRel) / sqrt(nrow(dataVarRatioFixedCostAggr)),1)
ciTotFix = round(
  qt(0.975,df=nrow(dataVarRatioFixedCostAggr)-1)*sd(dataVarRatioFixedCostAggr$comNetRel)/sqrt(nrow(dataVarRatioFixedCostAggr)),
  1
)

dataVarRatioFixedCostMean = aggregate(
  x=dataVarRatioFixedCost,
  comNetRel ~ month,
  mean
)


netVarRatioFixedCost = ggplot(
  dataVarRatioFixedCost,
  aes(x=month,
      y=comNetRel
  )
) +
  geom_line(
    aes(group=rep),
    color="blue",
    alpha=0.2
  ) +
  geom_line(
    data=dataVarRatioFixedCostMean,
    aes(
      x=month,
      y=comNetRel
    ),
    color="magenta",
    linewidth=1
  ) +
  # geom_point(shape=1) +
  # scale_y_continuous(
  #   label=dollar_format()) +
  scale_x_continuous(
    breaks=monthsVec) +
  geom_label(
    data=data.frame(
      month=12,
      height=max(dataVarRatioFixedCost$comNetRel),
      text=paste0(
        meanTotFix,
        "\u00b1",
        ciTotFix,
        "%"
      )
    ),
    aes(x=month,
        y=height,
        label=text
    ),
    nudge_x=-0.9,
    nudge_y=-0.4,
    size=2,
    color="magenta"
  ) +
  labs(x="Month",
       y="Net revenue (% initial CAC)") +
  plotTheme2 +
  theme(panel.grid.minor=element_blank())
netVarRatioFixedCost

ggsave(
  filename="output/netVarRatioFixedCost.pdf",
  width=8,
  height=5,
  units="cm"
)


## dynamic pricing simulation with variable riders:drivers ratio, initial 166:1, fixed number of replenished, variable optimum driver cost as a function of riders:drivers ratio ####

# the driver rides model
rideChargeRelDF = data.frame(
  charged=c(3, 6),
  rides=c(154, 100)
)
rideChargeModel = lm(
  rides ~ charged,
  data=rideChargeRelDF
)

# the quit rate model
driverQuitRelDF = data.frame(
  charged=c(3, 6),
  quit=c(0.09, 0.2)
)
driverQuitModel = lm(
  quit ~ charged,
  data=driverQuitRelDF
)

# riderCost = 25 # the cost of a ride for a rider
happyRiderQuitRate = 0.1 # the monthly rate of a rider who has been served so far 
sadRiderQuitRate = 0.33 # the monthly rate of a rider who has not been served at least once in the past
epsilon = as.numeric(rideChargeModel$coefficients[1])
delta = as.numeric(rideChargeModel$coefficients[2])
initAcqDrivers = 100 # will it be a variable later?
initAcqRiders = 16600 # will it be a variable later?
replDrivers = 7
replRiders = 1660
repsCount = 500
dataVarRatioVarCost = NULL
for (rep in 1:repsCount)
{
  print(rep)
  
  companyMoney = 0 # the budget of the company
  drivers = 0 # the initial number of drivers
  riders = 0 # the initial number of riders
  monthsVec = 1:12

  # at the end of the previous month, acquisitions are assumed to have occurred
  acqDrivers = initAcqDrivers # this for testing; 
  acqRiders = initAcqRiders # this for testing;
  initCAC = (acqDrivers * driverAC) + (acqRiders * riderAC)
  companyMoney = companyMoney - initCAC # update the company's money with the CAC
  happyRiders = rep(1, acqRiders)
  
  # the monthly loop
  for(month in monthsVec)
  {
    drivers = drivers + acqDrivers
    riders = riders + acqRiders
 
    ratio = max(0, riders / drivers)
    driverCost = max(0, (ratio - epsilon) / delta)
     
    driverRideRate = as.numeric(round(predict(rideChargeModel, data.frame(charged=driverCost)))) # the monthly rate of rides
    driverQuitRate = max(0, as.numeric(predict(driverQuitModel, data.frame(charged=driverCost)))) # the monthly rate of driver quit
    
    # assuming that each acquired rider will call once per month
    set.seed(rep + month)
    
    # calls during the month
    yesRides = min( # the number of answered calls
      riders, # if there are enough drivers, all riders are answered
      drivers * driverRideRate
    )
    noRides = riders - yesRides
    noRiders = sample( # select randomly
      x=1:riders, # which of the riders
      size=noRides, # were not answered
      replace=FALSE
    )
    happyRiders[noRiders] = 0 # steal the happiness from the no riders
    
    # update company's money with the profit from the rides
    comEarnings = (yesRides * driverCost)
    companyMoney = companyMoney + comEarnings
    
    # driver quits 
    driverQuits = rbinom( # randomly select
      n=drivers, # for each driver
      size=1,
      prob=driverQuitRate # if (s)he quits
    )
    driverQuits[which(is.na(driverQuits))] = 0  # because currently the quit rate can be negative for small enough cost
    quittedDrivers = sum(driverQuits) # their number
    
    # rider quits 
    riderQuits = rbinom( # randomly select
      n=riders, # for each rider
      size=1,
      prob=ifelse( # if (s)he quits
        happyRiders==1, # depending on their happiness
        happyRiderQuitRate,
        sadRiderQuitRate
      )
    )
    quittedRiders = sum(riderQuits) # their number
    
    # update their numbers after they quit
    drivers = drivers - quittedDrivers
    riders = riders - quittedRiders
    # update the happyRiders
    happyRiders = happyRiders[which(riderQuits==0)]
    
    # end of month acquisition of new drivers and riders; must have a specific strategy (finally, it will be plain replacement of the lost customers)
    acqDrivers = max(0, round(rnorm(n=1, mean=quittedDrivers, sd=2))) # rpois(n=1, lambda=quittedDrivers) #replDrivers
    acqRiders = max(0, round(rnorm(n=1, mean=quittedRiders, sd=150))) # rpois(n=1, lambda=quittedRiders)#replRiders
    comLosings = (acqDrivers * driverAC) + (acqRiders * riderAC)
    companyMoney = companyMoney - comLosings # update the company's money with the CAC
    comNet = comEarnings - comLosings
    happyRiders = c(happyRiders, rep(1, acqRiders))
    
    dataVarRatioVarCost = rbind(
      dataVarRatioVarCost,
      data.frame(
        rep,
        month,
        driverCost,
        companyMoney,
        comMoneyPercInitCAC = companyMoney/initCAC*100,
        comEarnings,
        comLosings,
        comNet,
        comNetRel=comNet/initCAC*100,
        drivers,
        quittedDrivers,
        riders,
        quittedRiders,
        yesRate = yesRides / (yesRides+noRides) * 100
      )
    )
  }
}

# data preparation for plotting
dataVarRatioVarCostAggr = aggregate(
  x=dataVarRatioVarCost,
  comNetRel ~ rep,
  sum
)
meanTotVar = round(mean(dataVarRatioVarCostAggr$comNetRel), 1)
seTot = round(sd(dataVarRatioVarCostAggr$comNetRel) / sqrt(nrow(dataVarRatioVarCostAggr)),1)
ciTotVar = round(
  qt(0.975,df=nrow(dataVarRatioVarCostAggr)-1)*sd(dataVarRatioVarCostAggr$comNetRel)/sqrt(nrow(dataVarRatioVarCostAggr)),
  1
)

dataVarRatioVarCostMean = aggregate(
  x=dataVarRatioVarCost,
  comNetRel ~ month,
  mean
)

netVarRatioVarCost = ggplot(
  dataVarRatioVarCost,
  aes(x=month,
      y=comNetRel
  )
) +
  geom_line(
    aes(group=rep),
    color="blue",
    alpha=0.2
  ) +
  geom_line(
    data=dataVarRatioVarCostMean,
    aes(
      x=month,
      y=comNetRel
    ),
    color="magenta",
    linewidth=1
  ) +
  # geom_point(shape=1) +
  # scale_y_continuous(
  #   label=dollar_format()) +
  scale_x_continuous(
    breaks=monthsVec) +
  geom_label(
    data=data.frame(
      month=12,
      height=max(dataVarRatioVarCost$comNetRel),
      text=paste0(
        meanTotVar,
        "\u00b1",
        ciTotVar,
        "%"
      )
    ),
    aes(x=month,
        y=height,
        label=text
    ),
    nudge_x=-0.9,
    nudge_y=-0.6,
    size=2,
    color="magenta"
  ) +
  labs(x="Month",
       y="Net revenue (% initial CAC)") +
  plotTheme2 +
  theme(panel.grid.minor=element_blank())
netVarRatioVarCost

ggsave(
  filename="output/netVarRatioVarCost.pdf",
  width=8,
  height=5,
  units="cm"
)


## multi-panel (Figure 8 in main text)
netFixedVar = ggarrange(
  netVarRatioFixedCost,
  netVarRatioVarCost,
  labels=paste0("(", letters[1:2], ")"),
  label.args = list(hjust=-3.6, vjust=2, gp=gpar(fontsize=8))
)
netFixedVar

ggsave(
  plot=netFixedVar,
  filename="output/netFixedVar.pdf",
  width=8,
  height=10,
  units="cm"
)



## some mathematical derivations of the previous two (covered in Section 7.2 and Appendix A) ####

# checking the fixed pricing scheme for the mathematical model
ggplot(
  dataVarRatioFixedCost,
  aes(x=month,
      y=riders+quittedRiders # drivers + quittedDrivers # (riders+quittedRiders) / (drivers+quittedDrivers) # yesRate # drivers, riders
  )
) +
  geom_line(
    aes(group=rep),
    color="blue",
    alpha=0.2
  ) +
  plotTheme2

# checking the dynamic pricing scheme for the mathematical model
ggplot(
  dataVarRatioVarCost,
  aes(x=(riders+quittedRiders) / (drivers+quittedDrivers),
      y=quittedDrivers # drivers + quittedDrivers # (riders+quittedRiders) / (drivers+quittedDrivers) # yesRate # drivers, riders
  )
) +
  geom_point(
    aes(group=rep),
    color="blue",
    alpha=0.2
  ) +
  plotTheme2



## fixed pricing scheme for only the net revenue

# the scatterplot data
dataVarRatioFixedCostRatioScat2 = 
  data.frame(
    ratio=(dataVarRatioFixedCost$riders+dataVarRatioFixedCost$quittedRiders) / (dataVarRatioFixedCost$drivers+dataVarRatioFixedCost$quittedDrivers),
    net=dataVarRatioFixedCost$comNetRel
  )

## considering the poisson processes
# the changing ratio within 3 standard deviations of quitting happy riders
(16600-3*sqrt(16600*0.1))/100 
(16600+3*sqrt(16600*0.1))/100
# the changing ratio within 3 standard deviations of quitting drivers
(16600)/(100+3*sqrt(100*0.0654))
(16600)/(100-3*sqrt(100*0.0654))
# these show that the ratio fluctuates mainly due to the drivers (confirmed by the previous plots of this section about the ratio VS numbers of riders or drivers)

# the range of ratios
ratioVec2 = seq(
  min(dataVarRatioFixedCostRatioScat2$ratio), 
  max(dataVarRatioFixedCostRatioScat2$ratio), 
  0.05)

# when ratio is the min 125, assuming that riders are the same and mainly drivers fluctuate, 16600/D = 125 =>
16600/125
# when ratio is the max 225
16600/225
# again confirmed by the plot
# then assuming a quadratic passing these two extremes, plus the 100--166 point:
driversCountDF = data.frame(
  ratio=c(125, 166, 225),
  drivers=c(16600/125, 100, 16600/225)
)
driversCountMod = lm(drivers ~ poly(ratio, degree=2, raw=TRUE), data=driversCountDF)
# the number of drivers
driversVec = predict(driversCountMod, data.frame(ratio=ratioVec2))

# the number of quitted drivers
qDriversVec = 0.06543333*driversVec

# the answered riders
answeredRidersVec2 = pmin(
  16600, # when enough drivers, all riders answered
  16600 - (ratioVec2 - 166)* driversVec
)

# the quitted riders
qRidersVec = pmax(
  16600*0.1,
  16600*0.1 + 2*(ratioVec2-166)*driversVec*0.23 
)

# taking all together for the net revenue
netBound2 = (2.33*answeredRidersVec2 - driverAC*qDriversVec - riderAC*qRidersVec) / initCAC * 100
netBound2DF = data.frame(
  netBound=netBound2,
  ratio=ratioVec2
)

# plotting
dataVarRatioFixedCostRatioPlot2 = ggplot(
  dataVarRatioFixedCostRatioScat2,
  aes(x=ratio,
      y=net
  )
) +
  # geom_point(
  #   shape=1
  # ) +
  geom_bin2d(bins=20) +
  scale_fill_gradient(low="lavender", high="black", n.breaks=5) +
  geom_line(
    data=netBound2DF,
    aes(
      x=ratio,
      y=netBound
    ),
    color="magenta"
  ) +
  scale_x_continuous(
    breaks=c(115, 140, 165, 190, 215)#,
    # limits=c(125, 205)
  ) +
  scale_y_continuous(
    expand=expansion(add=c(0.7,0.1)),
    breaks=seq(-20, 20, 5)
  ) +
  labs(
    x="Monthly ratio of riders:drivers",
    y="Net revenue (% initial CAC)",
    fill="Frequency"
  ) +
  plotTheme2
dataVarRatioFixedCostRatioPlot2
ggsave(
  plot=dataVarRatioFixedCostRatioPlot2,
  filename="output/dataVarRatioFixedCostRatioPlot2.pdf",
  width=8,
  height=5,
  units="cm"
)


## dynamic pricing for only the net revenue

# the scatterplot's data 
dataVarRatioVarCostRatioScat2 = 
  data.frame(
    ratio=(dataVarRatioVarCost$riders+dataVarRatioVarCost$quittedRiders) / (dataVarRatioVarCost$drivers+dataVarRatioVarCost$quittedDrivers),
    net=dataVarRatioVarCost$comNetRel
  )
#

# the ratios
ratioVarVec2 = seq(
  min(dataVarRatioVarCostRatioScat2$ratio),
  max(dataVarRatioVarCostRatioScat2$ratio),
  0.05
)

# then again assuming the same quadratic curve passing the two extremes and the 100--166 reference:
driversCountVarDF = data.frame(
  ratio=c(125, 166, 225),
  drivers=c(16600/125, 100, 16600/225)
)
driversCountModVar = lm(drivers ~ poly(ratio, degree=2), data=driversCountVarDF)
driversVecVar = predict(driversCountModVar, data.frame(ratio=ratioVarVec2))

# the driver costs for each ratio
costAvec2 = (ratioVarVec2 - epsilon) / delta

# the driver quit rate model
driverQuitRelDF = data.frame(
  charged=c(3, 6),
  quit=c(0.09, 0.2)
)
driverQuitModel = lm(
  quit ~ charged,
  data=driverQuitRelDF
)
# the number of quitted drivers
qDriversVecVar = predict(
  driverQuitModel, 
  data.frame(charged=costAvec2)) * driversVecVar

# the answered riders
answeredRidersVec2Var = 16600

# the quitted riders
qRidersVecVar = answeredRidersVec2Var * 0.1

# taking all together for the net revenue
netVecVar2 = (costAvec2*answeredRidersVec2Var - driverAC*qDriversVecVar - riderAC*qRidersVecVar) / initCAC * 100
ratioFinVar2 = data.frame(
  ratio=ratioVarVec2,
  net=netVecVar2
)

# plotting
dataVarRatioVarCostRatioPlot2 = ggplot(
  dataVarRatioVarCostRatioScat2,
  aes(x=ratio,
      y=net
  )
) +
  # geom_point(
  #   shape=1
  # ) +
  geom_bin2d(bins=20) +
  scale_fill_gradient(low="lavender", 
                      high="black", 
                      # n.breaks=7,
                      breaks=seq(100,1000,200),) +
  geom_line(
    data=ratioFinVar2,
    aes(
      x=ratio,
      y=net
    ),
    color="magenta"
  ) +
  scale_x_continuous(
    breaks=c(135, 150, 165, 180, 195)#,
    # limits=c(125, 205)
  ) +
  scale_y_continuous(
    expand=expansion(add=c(0.8,0.3)),
    breaks=seq(-20, 20, 5)
  ) +
  labs(
    x="Monthly ratio of riders:drivers",
    y="Net revenue (% initial CAC)",
    fill="Frequency"
  ) +
  plotTheme2
dataVarRatioVarCostRatioPlot2
ggsave(
  plot=dataVarRatioVarCostRatioPlot2,
  filename="output/dataVarRatioVarCostRatioPlot2.pdf",
  width=8,
  height=5,
  units="cm"
)

## multi-panel (Figure 9 in main text)
netVarRatioPlot2 = ggarrange(
  dataVarRatioFixedCostRatioPlot2,
  dataVarRatioVarCostRatioPlot2,
  labels=paste0("(", letters[1:2], ")"),
  label.args = list(hjust=-3.6, vjust=2, gp=gpar(fontsize=8))
)
netVarRatioPlot2

ggsave(
  plot=netVarRatioPlot2,
  filename="output/netVarRatioPlot2.pdf",
  width=8,
  height=10,
  units="cm"
)


# calculating some stuff for the main text (the example in the paragraph before the last one of Section 7.2)
maxRat = 146
minRat = 186

maxCost = (maxRat-208)/-18
minCost = (minRat-208)/-18

maxQuit = (3.6*maxCost-2)/100
minQuit = (3.6*minCost-2)/100

maxDrivers = 0.004*maxRat*maxRat-1.8*maxRat+306.6
minDrivers = 0.004*minRat*minRat-1.8*minRat+306.6

maxDrivers*maxQuit
minDrivers*minQuit

maxCost*16600
maxDrivers*maxQuit*500
minCost*16600
minDrivers*minQuit*500


(20289-57178)#/57178*100
(1500-6500)#/6500*100
